package com.incubator.game.handler;

import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.MsgUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

/**
 * 加入房间请求
 *
 * @author johnc
 */
public class JoinRoomHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.JoinRoomRes_VALUE);

		Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
		if (player == null) {
			logger.info("创建房间失败，找不到玩家");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			response.setMessage("操作失败...");
			return;
		}

		// 判断参数
		int roomId = Integer.parseInt(request.getData().getOrDefault("roomId", "").toString());
		if (roomId <= 0) {
			logger.info("参数错误");
			response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
			response.setMessage("参数错误...");
			return;
		}

		// 房间是否存在
		Room room = RoomService.getInstance().getRoom(roomId);
		if (room == null) {
			logger.info("房间已解散");
			response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
			response.setMessage("操作失败，房间已解散...");
			return;
		}

		// 判断玩家绑定房间
		if (player.data.roomId != 0 && player.data.roomId != roomId) {
			logger.info("不可重复加入其他房间");
			response.setCode(CommonProto.Code.JOIN_ROOM_REPEATEDLY_ERR_VALUE);
			response.setMessage("操作失败，不可重复加入其他房间...");
			return;
		}

		// 加入房间
		room = RoomService.getInstance().joinRoom(room, player);

		// 玩家绑定房间
		player.data.roomId = room.data.roomId;

		// 正常返回
		response.setData(MsgUtil.roomToMessage(room, player, null));
	}
}
