package com.incubator.game.handler.jdgd;

import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.room.jdgd.JDGDRoom;
import com.incubator.game.player.Player;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.CardUtils;
import com.incubator.game.util.MsgUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

import java.util.*;

/**
 * 经典掼蛋 出牌请求
 *
 * @author johnc
 */
public class JDGDDisCardHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) throws Exception {
        response.setCmd(CommonProto.Cmd.PlayDisCardsRes_VALUE);

        Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
        if (player == null) {
            logger.info("创建房间失败，找不到玩家");
            response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
            response.setMessage("操作失败...");
            return;
        }

        // 判断玩家绑定房间
        if (player.data.roomId <= 0) {
            logger.info("准备失败，玩家不在房间");
            response.setCode(CommonProto.Code.PLAYER_ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，玩家不在房间...");
            return;
        }

        // 玩家房间是否存在
        JDGDRoom room = (JDGDRoom) RoomService.getInstance().getRoom(player.data.roomId);
        if (room == null) {
            logger.info("玩家房间已解散");
            response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，玩家房间已解散...");
            return;
        }

        // 当前状态是否可操作
        if (room.data.state != 2 || player.data.state != 2) {
            logger.info("房间或玩家不在游戏中状态...");
            response.setCode(7);
            response.setMessage("操作失败，房间或玩家不在游戏中状态...");
            return;
        }

        // 当前可操作玩家是否自己
        if (!room.data.currentPlayer.getId().equals(player.getId())) {
            logger.info("当前不是自己出牌回合...");
            response.setCode(8);
            response.setMessage("操作失败，当前不是自己出牌回合...");
            return;
        }

        // 判断参数
        int[] disCardList = (int[]) request.getData().get("disCardList");
        if (disCardList == null || disCardList.length <= 0) {
            logger.info("参数错误");
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("参数错误...");
            return;
        }

        // 判断手牌
        if (!CardUtils.contains(player.data.remainCards, disCardList)) {
            logger.info("参数错误");
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("参数错误，玩家出的牌不在手上...");
            return;
        }

        // 出牌
        room.playerDisCard(player, disCardList);

        // 正常返回
        response.setData(MsgUtil.roomToMessage(room, player, room.data.currentPlayer));
    }
}
