package com.incubator.game.listener;

import com.incubator.core.net.ws.Message;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.GGame;
import com.incubator.common.net.Connection;
import com.incubator.game.player.GPlayer;
import com.incubator.core.net.handler.listener.GameServerConnectionListener;
import com.incubator.message.proto.CommonProto;
import io.netty.buffer.ByteBuf;

/**
 * 公共接口
 * 
 * @author johnc
 *
 */
public class PublicListener extends GameServerConnectionListener {

	@Override
	public void connectionOpened(Connection connection) {
		HeartConfig heart = new HeartConfig();
		connection.setAttribute(HeartConfig, heart);
		setHeartEnable(GGame.heartEnable);
		GGame.publicClients.add(connection);
		connection.setAttribute(LastReadTimeMills, System.currentTimeMillis()); // 设置最后心跳时间
	}

	@Override
	public void connectionClosed(Connection connection) {
		//连接关闭操作
		try {
			GPlayer player = GGame.onlinePlayers.getOrDefault(connection.getPlayerId(), null);
			if (player != null && player.getSession() == connection) {
				player.doLogout();
			}
		} catch (Exception e) {
			logger.error("玩家退出...");
		} finally {
			GGame.publicClients.remove(connection);
			if (connection.getPlayerId() != null && !GGame.onlinePlayers.isEmpty()) {
				GGame.onlinePlayers.remove(connection.getPlayerId());
			}
		}
	}

	@Override
	public void messageArrived(Connection conn, Object msg) {
		try {
			if (msg instanceof ByteBuf) {
				ByteBuf byteBuf = (ByteBuf) msg;

//				conn.setAttribute(LastReadTimeMills, System.currentTimeMillis());// 设置最后心跳时间
//				resetHeartSeq(conn);// 设置心跳
//				if (isHeartReq(msg)) {
//					sendHeartRsp(conn);
//					return;
//				}

				byte[] bytes = new byte[byteBuf.readableBytes()];
				byteBuf.readBytes(bytes);

				WSRequest request = new WSRequest(bytes);

				NetHandler handler = GGame.handlers.get(request.getCmd());
				if (handler != null) {
					// 过滤一下心跳消息
					if (request.getCmd() != CommonProto.Cmd.HeartBeatReq_VALUE) {
						logger.info("收到消息 : cmd={}, 请求内容 : {}", request.getCmd(), request.toJson());
					}
					WSResponse response = new WSResponse();
					handler.onDate(conn, request, response);
					conn.writeAndFlush(response.toBytes());
					if (request.getCmd() != CommonProto.Cmd.HeartBeatReq_VALUE) {
						logger.info("返回消息 : cmd={}, 返回内容 : {}", request.getCmd(), response.toJson());
					}
				} else {
					logger.error("消息号错误... cmd={}", request.getCmd());
					conn.close();
				}
			}
		} catch (Exception e) {
			logger.error("...数据解析有错...", e);
			conn.close();
		}
	}

	@Override
	public boolean doIsHeartReq(Object msg) {
		try {
			ByteBuf byteBuf = (ByteBuf) msg;
			byte[] bytes = new byte[byteBuf.readableBytes()];
			byteBuf.readBytes(bytes);
//			MessageUnpacker unpacker = MessagePack.newDefaultUnpacker(bytes);
//			WSRequest request = super.parseWSRequest(unpacker);
			WSRequest request = new WSRequest(bytes);
			return request.getCmd() == CommonProto.Cmd.HeartBeatReq_VALUE;
		} catch (Exception e) {
			logger.error("心跳请求解析失败", e);
			return false;
		}
	}

	@Override
	public void doSendHeartReq(Connection conn) {// 发送心跳请求
        try {
            long now = System.currentTimeMillis();
            long lastTime = (long) conn.getAttribute(LastReadTimeMills);
            if (now - lastTime > 10000L) {
//                logger.info("玩家网络断了：" + conn.getUid());
//                conn.close();
            } else {
//                HeartbeatPush push = new HeartbeatPush();
//                push.setTime(System.currentTimeMillis());
//                Message msg = new Message(ProtoCommon.MsgId.S2C_Heartbeat, push.toJson());
//                msg.setCode(ProtoCommon.Code.OK_VALUE);
//                conn.writeAndFlush(super.packMessage(msg));
            }
        } catch (Exception e) {
			logger.info("发送心跳请求出错：" + conn.getUid() + e.getMessage());
        }

        // {//借助心跳完成一些任务
		// if(connection.getId() != null && connection.getId() > 0) {
		// logger.info("心跳用户：{}" , connection.getId());
		// }
		// }
	}

	@Override
	public void doSendHeartRsp(Connection connection) {// 发送心跳回复
		 logger.info("＝＝＝＝＝＝＝＝＝＝＝＝＝我收到了：" + connection.getPlayerId());
	}
}
