package com.incubator.game.room;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.game.data.model.RoomPO;
import com.incubator.game.room.jdgd.JDGDRoom;
import com.incubator.game.room.jdgd.JDGDSchedule;
import org.slf4j.Logger;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 房间实体类
 */
public class Room {

    protected Logger logger = Log4jUtil.getLogger(getClass());

    /** 创建一个单线程的 ScheduledExecutorService **/
    private ScheduledExecutorService scheduler = Executors.newSingleThreadScheduledExecutor();

    /** 房间数据 **/
    public RoomPO data;

    public Room() {
        this.data = new RoomPO();
    }

    /**
     * 开启房间主线程任务
     */
    public void startRoomTask() {
        // 启动一个每秒运行一次的任务
        // 初始延迟为0，后续每隔1秒执行一次
        // 例如，这里可以检查游戏状态、广播信息、或者更新某些数据
        this.scheduler.scheduleAtFixedRate(this::performRoomLogic, 0, 1, TimeUnit.SECONDS);
    }

    private void performRoomLogic() {
        switch (this.data.type) {
            case 1:
                if (this instanceof JDGDRoom) {
                    JDGDSchedule.getInstance().doUpdate((JDGDRoom) this);
                }
                break;
        }
    }

    /**
     * 关闭房间主线程任务
     */
    public void stopRoomTask() {
        // 关闭调度器
        if (!this.scheduler.isShutdown()) {
            this.scheduler.shutdown();
            try {
                if (!this.scheduler.awaitTermination(5, TimeUnit.SECONDS)) {
                    this.scheduler.shutdownNow();
                }
            } catch (InterruptedException e) {
                this.scheduler.shutdownNow();
            }
            this.scheduler = null;
        }
    }
}
