package com.incubator.game.room;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.util.RandomUtil;
import com.incubator.game.player.Player;
import com.incubator.game.room.jdgd.JDGDRoom;
import com.incubator.game.util.MsgUtil;
import com.incubator.message.proto.CommonProto;
import org.slf4j.Logger;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 房间 管理类
 */
public class RoomService {

    protected Logger logger = Log4jUtil.getLogger(getClass());

    private static RoomService instance;

    public Map<Integer, Room> roomMap = new ConcurrentHashMap<>();

    public synchronized static RoomService getInstance() {
        if (instance == null) {
            instance = new RoomService();
        }
        return instance;
    }

    /**
     * 初始化
     */
    public void init() {

    }

    /**
     * 每日刷新
     */
    public void refreshNewDay() {

    }

    /**
     * 生成唯一6位数房间号
     *
     * @return
     */
    public int randomRoomNumber() {
        int roomNumber;
        // 循环生成，直到找到一个唯一的房间号
        do {
            roomNumber = 100000 + RandomUtil.random(900000); // 生成 100000 到 999999 之间的随机数
        } while (roomMap.containsKey(roomNumber));
        return roomNumber;
    }

    /**
     * 获取房间
     *
     * @param roomId
     * @return
     */
    public Room getRoom(int roomId) {
        return this.roomMap.getOrDefault(roomId, null);
    }

    /**
     * 获取玩家房间
     *
     * @param playerId
     * @return
     */
    public Room getRoomByPlayerId(String playerId) {
        for (Room room : this.roomMap.values()) {
            if (room != null && room.data.playerIds != null && room.data.playerIds.containsKey(playerId)) {
                return room;
            }
        }
        return null;
    }

    /**
     * 创建房间
     *
     * @param player
     * @param type
     * @return
     */
    public Room creatRoom(Player player, int type) {
        Room room = null;

        switch (type) {
            case 1:
                room = new JDGDRoom();
                break;
        }

        if (room == null) {
            return null;
        }

        room.data.roomId = randomRoomNumber();
        room.data.type = 1;
        room.data.ownerId = player.getId();
        room.data.maxNum = 4;
        if (!room.data.playerIds.containsKey(player.getId())) {
            room.data.playerIds.put(player.getId(), player);
        }
        room.data.playerMap.put(0, player);

        // 加入缓存
        this.roomMap.put(room.data.roomId, room);

        return room;
    }

    /**
     * 加入房间
     *
     * @param room
     * @param player
     * @return
     */
    public Room joinRoom(Room room, Player player) {
        if (!room.data.playerIds.containsKey(player.getId())) {
            room.data.playerIds.put(player.getId(), player);
        }
        int index = 0;
        for (Player tmpPlayer : room.data.playerIds.values()) {
            if (tmpPlayer != null) {
                tmpPlayer.data.pos = index;
                room.data.playerMap.put(index, tmpPlayer);
                index++;
            }
        }

        // 推送房间其他3个玩家
        for (Player tmPlayer : room.data.playerMap.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                Map<String, Object> data = MsgUtil.roomToMessage(room, tmPlayer, null);
                tmPlayer.receive(CommonProto.Cmd.RoomPlayerJoin_VALUE, data);
            }
        }

        return room;
    }

    /**
     * 删除房间
     *
     * @param roomId
     */
    public void remove(int roomId) {
        Room room = this.roomMap.getOrDefault(roomId, null);
        if (room == null) {
            return;
        }
        logger.debug("房间销毁...房间号 : {}", roomId);
        // 停止任务调度器
        room.stopRoomTask();
        room = null;
        this.roomMap.remove(roomId);
    }
}
