package com.incubator.game.room.jdgd;

import com.incubator.game.player.Player;
import com.incubator.game.room.GRoomInterface;
import com.incubator.game.room.Room;
import com.incubator.game.util.CardUtils;
import com.incubator.game.util.MsgUtil;
import com.incubator.message.proto.CommonProto;

import java.util.*;

/**
 * 经典掼蛋房间
 */
public class JDGDRoom extends Room implements GRoomInterface {

    /**
     * 掼蛋对应的数值
     * 0x03-0x0f 方片（♦️）3(3) 4(4) 5(5) 6(6) 7(7) 8(8) 9(9) 10(10) J(11) Q(12) K(13) A(14) 2(15)
     * 0x13-0x1f 梅花（♣️）3(19) 4(20) 5(21) 6(22) 7(23) 8(24) 9(25) 10(26) J(27) Q(28) K(29) A(30) 2(31)
     * 0x23-0x2f 红桃（♥️）3(35) 4(36) 5(37) 6(38) 7(39) 8(40) 9(41) 10(42) J(43) Q(44) K(45) A(46) 2(47)
     * 0x33-0x3f 黑桃（♠️）3(51) 4(52) 5(53) 6(54) 7(55) 8(56) 9(57) 10(58) J(59) Q(60) K(61) A(62) 2(63)
     * 0xef      小王(239)
     * 0xff      大王(255)
     */
    public static Integer[] values = {
            0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
            0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f,
            0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e, 0x2f,
            0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e, 0x3f,
            0xef,
            0xff
    };

    public JDGDRoom() {
        this.initCardList();
    }

    /**
     * 初始化牌库
     */
    @Override
    public void initCardList() {
        for (int i = 0; i < 2; i++) {
            this.data.cardList.addAll(Arrays.asList(values));
            Collections.shuffle(this.data.cardList);
        }
    }

    /**
     * 玩家准备
     *
     * @param player
     */
    public void playerReady(Player player) {
        if (player.data.state == 1) {
            return;
        }

        // 设置玩家状态
        player.data.state = 1;

        // 推送房间其他3个玩家
        for (Player tmPlayer : this.data.playerMap.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                Map<String, Object> data = MsgUtil.roomToMessage(this, tmPlayer, null);
                tmPlayer.receive(CommonProto.Cmd.ReadyStateUpdate_VALUE, data);
            }
        }
    }

    /**
     * 检测是否可以开始 3秒倒计时开始
     *
     * @return
     */
    @Override
    public boolean checkReadyStart() {
        if (!this.isStart() || this.data.state != 0) {
            return false;
        }

        // 广播3秒倒计时
        for (Player tmPlayer : this.data.playerMap.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = MsgUtil.roomToMessage(this, tmPlayer, null);
                data.put("state", 1);
                data.put("time", 3);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }

        return true;
    }

    /**
     * 是否玩家都已准备,可开始
     *
     * @return
     */
    @Override
    public boolean isStart() {
        int cnt = 0;
        for (Player player : this.data.playerMap.values()) {
            if (player != null && player.data.state == 1) {
                cnt += 1;
            }
        }
        return cnt >= this.data.maxNum;
    }

    /**
     * 给玩家发牌，每人27张
     */
    @Override
    public void doSendCard() {
        for (Player player : this.data.playerMap.values()) {
            if (player != null) {
                // 设置用户状态
                player.data.state = 2;
                // 发牌
                for (int i = 0; i < 27; i++) {
                    int card = this.data.cardList.get(0);
                    player.data.remainCards = CardUtils.add(player.data.remainCards, card);
                    this.data.cardList.remove(0);
                }
            }
        }
    }

    /**
     * 游戏开局
     */
    @Override
    public void doStart() {
        this.data.curRound += 1;
        // 定庄
        this.data.zhuangPos = 0;
        // 设置当前操作玩家
        this.data.currentPlayer = this.data.playerMap.get(this.data.zhuangPos);
        // 发牌
        this.doSendCard();
        // 广播
        for (Player tmPlayer : this.data.playerMap.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = MsgUtil.roomToMessage(this, tmPlayer, this.data.currentPlayer);
                data.put("state", 2);
                data.put("time", 20);
                CardUtils.printCards(tmPlayer.data.remainCards);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }
    }

    /**
     * 根据当前玩家位置, 获取下一个操作玩家位置
     *
     * @param pos
     * @return
     */
    public Integer[] _getSorcPos(int pos) {
        Integer nextpos[] = null;
        if (pos == 3) {
            nextpos = new Integer[] { 0, 1, 2 };
        } else if (pos == 2) {
            nextpos = new Integer[] { 3, 0, 1 };
        } else if (pos == 1) {
            nextpos = new Integer[] { 2, 3, 0 };
        } else if (pos == 0) {
            nextpos = new Integer[] { 1, 2, 3 };
        }
        return nextpos;
    }

    /**
     * 玩家出牌
     *
     * @param player
     * @param disCardList
     */
    public void playerDisCard(Player player, int[] disCardList) {
        // 是否自己回合
        if (!this.data.currentPlayer.getId().equals(player.getId())) {
            return;
        }
        if (CardUtils.contains(player.data.remainCards, disCardList)) {
            logger.info("出牌...玩家ID:{}, 位置:{}, 手牌大小:{}, 手牌信息: {}", player.getId(), player.data.pos, player.data.remainCards.length, Arrays.toString(player.data.remainCards));
            // 从手牌中移除
            for (Integer card : disCardList) {
                player.data.remainCards = CardUtils.remove(player.data.remainCards, card);
            }
            // 设置出牌堆
            this.data.curDisCardList = disCardList;
            // 设置当前出牌的玩家
            this.data.disCardPlayer = player;
            // 设置结算数据
            if (player.data.remainCards.length == 0) {
                player.data.state = 3;
                this.data.result.put(this.data.result.size() + 1, player.getId());
            }
            // 设置下一个操作玩家
            Integer[] nextPos = this._getSorcPos(player.data.pos);
            this.data.currentPlayer = this.data.playerMap.get(nextPos[0]);
            // 广播其他3人
            for (Player tmPlayer : this.data.playerMap.values()) {
                if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                    Map<String, Object> data = MsgUtil.roomToMessage(this, tmPlayer, this.data.currentPlayer);
                    tmPlayer.receive(CommonProto.Cmd.OtherPlayCards_VALUE, data);
                }
            }
            // 3人手牌出完，直接结算
            if (this.data.result.size() >= 3) {
                // 结束
                this.gameOver();
            }
        }
    }

    /**
     * 牌局结算
     */
    public void settlement() {
        logger.debug("牌局结算...roomId : {}", this.data.roomId);
        // 广播结算
        for (Player tmPlayer : this.data.playerMap.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = MsgUtil.roomToMessage(this, tmPlayer, null);
                // 结算信息
                data.put("settlementInfo", this.data.result);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }
    }

    /**
     * 游戏结束
     */
    public void gameOver() {
        this.data.state = 3;
        this.data.time = 60;
        this.data.flag = false;
        for (Player player : this.data.playerMap.values()) {
            if (player != null) {
                player.data.state = 3;
            }
        }
        // 结算
        this.settlement();
        logger.debug("牌局结束...roomId : {}", this.data.roomId);
    }
}
