package com.incubator.game.room.jdgd;

import com.incubator.common.log4j.Log4jUtil;
import org.slf4j.Logger;

/**
 * 经典掼蛋主线程逻辑
 */
public class JDGDSchedule {

    protected Logger logger = Log4jUtil.getLogger(getClass());

    private static JDGDSchedule instance;

    public synchronized static JDGDSchedule getInstance() {
        if (instance == null) {
            instance = new JDGDSchedule();
        }
        return instance;
    }

    public void doUpdate(JDGDRoom room) {
        if (room == null) {
            logger.error("经典掼蛋 : 主线程逻辑找不到房间实体...");
            return;
        }

        switch (room.data.state) {
            case 0:
                // 等待状态
                if (room.checkReadyStart()) {
                    room.data.state = 1;
                    room.data.flag = false;
                    room.data.time = 0;
                }

                // 120秒未开始直接解散
//                if (room.data.time >= 120) {
//                    logger.debug("解散房间...");
//                    RoomService.getInstance().remove(room.data.roomId);
//                }
                break;
            case 1:
                // 开局
                if (!room.data.flag) {
                    //开始
                    if (room.data.time >= 3) {
                        room.doStart();
                        room.data.state = 2;
                        room.data.flag = true;
                        room.data.time = 0;
                        logger.debug("检测: 房间号:{}, 房间人数:{}, 当前局数:{}, 状态:开局...VS动画...", room.data.roomId, room.data.playerMap.size(), room.data.curRound);
                    }
                }
                else {
                    if (room.data.time >= 4) {
                        room.data.state = 2;
                        room.data.flag = false;
                        room.data.time = 0;
                    }
                }
                break;
            case 2:
                // 进行中
                if (!room.data.flag) {
                    room.data.flag = true;
                    room.data.time = 0;
                } else {
//                    logger.debug("强制操作...");
                }
                break;
            case 3:
                // 结束
                if (!room.data.flag) {
                    room.data.flag = true;
                    room.data.time = 0;
                    logger.debug("游戏结束: 房间号 : {}, 房间人数 : {}, 当前局数 : {}", room.data.roomId, room.data.playerMap.size(), room.data.curRound);
                } else {

                }
                break;
        }

        room.data.time += 1;
    }

}
